import { Camera, Canvas, Children, Circle, Commands, Cursor, Ellipse, FillGradient, FillSolid, Name, Opacity, Parent, Path, Polyline, Rect, Renderable, Selected, Stroke, System, Text, Transform, Visibility, } from '@infinite-canvas-tutorial/ecs';
import { Container } from '../components';
import { Event } from '../event';
import { API, pendingCanvases } from '../API';
export class InitCanvasSystem extends System {
    constructor() {
        super();
        this.commands = new Commands(this);
        this.query((q) => q.using(Container, Canvas, Camera, Name, Cursor, Transform, Parent, Children, Renderable, Visibility, FillSolid, FillGradient, Stroke, Circle, Ellipse, Rect, Polyline, Path, Text, Selected, Opacity).write);
    }
    execute() {
        if (pendingCanvases.length) {
            pendingCanvases.forEach(({ container, canvas, camera }) => {
                const api = new API(container, this.commands);
                container.apiProvider.setValue(api);
                api.createCanvas(canvas);
                api.createCamera(camera);
                api.getAppState = container.getAppState.bind(container);
                api.setAppState = container.setAppState.bind(container);
                api.getNodes = container.getNodes.bind(container);
                api.setNodes = container.setNodes.bind(container);
                this.commands.execute();
                container.dispatchEvent(new CustomEvent(Event.READY, { detail: api }));
            });
            pendingCanvases.length = 0;
        }
    }
}
//# sourceMappingURL=InitCanvas.js.map